<?php

namespace Growmatik\Admin;

class Email_Consent {

	public function __construct() {
		if ( ! Integration::is_integrated() ) {
			return;
		}

		$this->add_default_settings();
		add_action( 'wp_ajax_growmatik_save_settings', array( $this, 'save_settings' ) );

		$options = get_option( 'growmatik_consents' );

		if ( $options['signup'] ) {
			// WP Registration form.
			add_action( 'register_form', array( $this, 'add_registration_form_field' ) );
			add_action( 'user_register', array( $this, 'save_registration_form_field' ) );
			add_action( 'login_enqueue_scripts', array( $this, 'styles' ) );
		}

		if ( $options['wc_signup'] ) {
			add_action( 'woocommerce_register_form', array( $this, 'add_wc_register_form_field' ) );
			add_action( 'woocommerce_created_customer', array( $this, 'save_wc_register_form_field' ) );
		}

		if ( $options['wc_checkout'] ) {
			add_filter( 'woocommerce_order_button_html', array( $this, 'add_wc_checkout_field' ), 10, 1 );
			add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'save_wc_checkout_field' ) );
		}

		if ( ! empty( $options['edd_checkout'] ) ) {
			add_action( 'edd_purchase_form_user_info_fields', array( $this, 'add_edd_checkout_field' ) );
			add_filter( 'edd_payment_meta', array( $this, 'save_edd_checkout_field' ), 999 );
		}
	}

	public function save_settings() {
		$nonce = isset( $_POST['nonce'] ) ? sanitize_text_field( wp_unslash( $_POST['nonce'] ) ) : '';

		if ( ! wp_verify_nonce( $nonce, 'growmatik_ajax' ) ) {
			wp_send_json_error();
		}

		$options = array(
			'signup'       => (int) filter_var( $_POST['growmatik_consent_signup'], FILTER_VALIDATE_BOOLEAN ), //phpcs:ignore
			'wc_signup'    => (int) filter_var( $_POST['growmatik_consent_wc_signup'], FILTER_VALIDATE_BOOLEAN ), //phpcs:ignore
			'wc_checkout'  => (int) filter_var( $_POST['growmatik_consent_wc_checkout'], FILTER_VALIDATE_BOOLEAN ), //phpcs:ignore
			'edd_checkout' => (int) filter_var( $_POST['growmatik_consent_edd_checkout'], FILTER_VALIDATE_BOOLEAN ), //phpcs:ignore
			'consent_text' => sanitize_text_field( wp_unslash( $_POST['growmatik_consent_statement'] ) ), //phpcs:ignore
		);

		$options['consent_text'] = str_replace( '{{sitename}}', get_bloginfo( 'name' ), $options['consent_text'] );

		update_option( 'growmatik_consents', $options );

		wp_send_json_success();
	}

	// Add EDD.
	public function add_edd_checkout_field() {
		$this->show_checkbox( 'growmatik_consent_edd_checkout' );
	}

	// Save EDD.
	public function save_edd_checkout_field( $payment_meta ) {
		if ( did_action( 'edd_pre_process_purchase' ) && ! empty( $payment_meta ) ) {

			$payment_meta['growmatik_email_consent'] = 'denied';

			if ( isset( $_POST['growmatik_consent_edd_checkout'] ) && 'on' === 'growmatik_consent_edd_checkout' ) { // phpcs:ignore
				$payment_meta['growmatik_email_consent'] = 'granted';
			}

			if ( isset( $payment_meta['user_info']['id'] ) ) {
				update_user_meta( $payment_meta['user_info']['id'], 'growmatik_email_consent', $payment_meta['growmatik_email_consent'] );
			}
		}

		return $payment_meta;
	}

	// Add WC checkout.
	public function add_wc_checkout_field( $html ) {
		return $this->show_checkbox( 'growmatik_consent_wc_checkout' ) . $html;
	}

	// Save WC checkout.
	public function save_wc_checkout_field( $order_id ) {

		$order   = new \WC_Order( $order_id );
		$user_id = $order->get_user_id();

		if ( isset( $_POST['growmatik_consent_wc_checkout'] ) && 'on' === $_POST['growmatik_consent_wc_checkout'] ) { //phpcs:ignore
			update_user_meta( $user_id, 'growmatik_email_consent', 'granted' );
		} else {
			update_user_meta( $user_id, 'growmatik_email_consent', 'denied' );
		}
	}

	// Add WC register.
	public function add_wc_register_form_field() {
		$this->show_checkbox( 'growmatik_consent_wc_signup' );
	}

	// Save WC register.
	public function save_wc_register_form_field( $user_id ) {
		if ( isset( $_POST['growmatik_consent_wc_signup'] ) && 'on' === $_POST['growmatik_consent_wc_signup'] ) { //phpcs:ignore
			update_user_meta( $user_id, 'growmatik_email_consent', 'granted' );
		} else {
			update_user_meta( $user_id, 'growmatik_email_consent', 'denied' );
		}
	}

	// Add WP Signup.
	public function add_registration_form_field() {
		$this->show_checkbox( 'growmatik_consent_signup' );
	}

	// Save WP Signup.
	public function save_registration_form_field( $user_id ) {
		if ( isset( $_POST['growmatik_consent_signup'] ) && 'on' === $_POST['growmatik_consent_signup'] ) { // phpcs:ignore
			update_user_meta( $user_id, 'growmatik_email_consent', 'granted' );
		} else {
			update_user_meta( $user_id, 'growmatik_email_consent', 'denied' );
		}
	}

	// Avoids multiple checks for option existence.
	public function add_default_settings() {
		$defaults = array(
			'signup'       => 0,
			'wc_signup'    => 0,
			'wc_checkout'  => 0,
			'edd_checkout' => 0,
			'consent_text' => __( 'I\'d like to subscribe to {{sitename}} newsletter to get product updates & news, weekly digest, and more.', 'growmatik' ),
		);

		add_option( 'growmatik_consents', $defaults );
	}

	public static function remove_consent_settings() {
		delete_option( 'growmatik_consents' );
	}

	// Generate checkbox.
	private function show_checkbox( $name = '' ) {
		$options = get_option( 'growmatik_consents' );
		?>
		<p class="growmatik-consent">
			<label for="<?php echo esc_attr( $name ); ?>">
				<input
				type="checkbox"
				id="<?php echo esc_attr( $name ); ?>"
				name="<?php echo esc_attr( $name ); ?>">
				<?php echo esc_html( $options['consent_text'] ); ?>
			</label>
		</p>
		<?php
	}

	public function styles() {
		echo '
		<style type="text/css">
		#registerform {
			display: flex;
			flex-direction: column;
		}
		p.submit {
			order: 90;
		}
		p.growmatik-consent {
			order: 80;
		}
		</style>
		';
	}

}
