<?php
/**
 * Contains the Session_Manager class.
 *
 * @package Growmatik
 */

namespace Growmatik;

/**
 * Class Session_Manager
 *
 * Retrieves analytics session details from cookies.
 *
 * @package Growmatik
 */
class Session_Manager {
	/**
	 * The parsed contents of the cookie
	 *
	 * @var object
	 */
	private $cookie;

	/**#@+
	 * If any of the $override_user/$override_commerce are used, the FE won't pick up and update
	 * $this->cookie->overrideUser/overrideCommerce until the next page refresh.
	 * Therefore for the duration of current page, we need to override it manually. This is only useful when one of
	 * `get_effective_*` functions of this class are explicitly called.
	 *
	 * Normal access to WP_User object properties won't include these, e.g first name, etc.
	 *
	 * @var object
	 */
	private static $override_user     = null;
	private static $override_commerce = null;
	/**#@-*/

	/**
	 * Session_Manager constructor
	 */
	public function __construct() {
		$this->get_analytics_cookie();

		if ( self::$override_user ) {
			$this->cookie->overrideUser = self::$override_user;
		}

		if ( self::$override_commerce ) {
			$this->cookie->overrideCommerce = self::$override_commerce;
		}
	}

	/**
	 * Get the permanent client id of the visitor
	 *
	 * @return string|null
	 */
	public function get_client_id() {
		return isset( $this->cookie->clientId ) ? $this->cookie->clientId : null;
	}

	/**
	 * Get the temporary session id of the visitor
	 *
	 * @return string|null
	 */
	public function get_session_id() {
		return isset( $this->cookie->sessionId ) ? $this->cookie->sessionId : null;
	}

	/**
	 * Put a reset session id request at the footer which will be picked up by the script.
	 *
	 * Does NOT change the session ID for the duration of the current request. But if FE is going to use it again,
	 * it should reset, like {@see $override_user} / {@see $override_commerce}.
	 *
	 * @param $session_id
	 * @param string     $reason
	 */
	public function reset_session_id( $session_id, $reason ) {
		$session_reset = array(
			'reason'    => $reason,
			'sessionId' => $session_id,
		);

		$echo_reset = function () use ( $session_reset ) {
			wp_localize_script( 'growmatik-if-then', '_growmatikSessionReset', $session_reset );
		};

		add_action( 'wp_enqueue_scripts', $echo_reset, 8 );
	}

	/**
	 * Override the user properties of the visitor; e.g upgrade them to a Guest Customer
	 *
	 * @param array $props Properties to override; only non-empty props will be overridden.
	 * @param bool $reset If set to false, adds new properties to te override cookie without clearing it.
	 */
	public function override_user_props( $props, $reset = true ) {
		$user_override = array_filter( $props );

		if ( ! $reset ) {
			// $this->cookie->overrideUser must be equal to self::$override
			$user_override = array_merge( (array) $this->cookie->overrideUser, $user_override );
		}

		$user_override = (object) $user_override;

		self::$override_user        = $user_override;
		$this->cookie->overrideUser = $user_override;

		$echo_override = function () use ( $user_override ) {
			wp_localize_script( 'growmatik-if-then', '_growmatikUserOverride', (array) $user_override );
		};

		add_action( 'wp_enqueue_scripts', $echo_override, 8 );
	}

	/**
	 * Override the commerce properties of the visitor; similar to {@see override_user_props()}
	 *
	 * @param string $platform_key
	 * @param array $props Properties to override; only non-empty props will be overridden.
	 * @param bool $reset If set to false, adds new properties to te override cookie without clearing it.
	 */
	public function override_commerce_props( $platform_key, $props, $reset = true ) {
		$commerce_override = array(
			// Do not use 'array_filter' in case of Ecommerce overrides;
			// all attributes have to be set for consistency.
			$platform_key => $props,
		);

		if ( ! $reset ) {
			// $this->cookie->overrideCommerce must be equal to self::$override
			$commerce_override = array_merge( (array) $this->cookie->overrideCommerce, $commerce_override );
		}

		$commerce_override = (object) $commerce_override;

		self::$override_commerce        = $commerce_override;
		$this->cookie->overrideCommerce = $commerce_override;

		$echo_override = function () use ( $commerce_override ) {
			wp_localize_script( 'growmatik-if-then', '_growmatikCommerceOverride', (array) $commerce_override );
		};

		add_action( 'wp_enqueue_scripts', $echo_override, 8 );
	}

	/**
	 * Gets the effective overridden user property of Guest Subscribers or Guest Customers.
	 * This is set by the FE script on popup form subscription, or by the plugin on Guest Checkout.
	 *
	 * @param string $property
	 *
	 * @return mixed|null
	 */
	public function get_effective_user_property( $property ) {
		if ( isset( $this->cookie->overrideUser->{$property} ) ) {
			return $this->cookie->overrideUser->{$property};
		}

		return null;
	}

	/**
	 * Similar to {@see get_effective_user_property()}
	 *
	 * @param string $property
	 *
	 * @return mixed|null
	 */
	public function get_effective_commerce_property( $property ) {
		if ( isset( $this->cookie->overrideCommerce->{$property} ) ) {
			return $this->cookie->overrideCommerce->{$property};
		}

		return null;
	}

	/**
	 * Parses the analytics cookie
	 *
	 * @return void
	 */
	private function get_analytics_cookie() {
		$this->cookie = json_decode( filter_input( INPUT_COOKIE, 'outstand' ) );
	}
}
