<?php

namespace Growmatik\Controller;

use Growmatik\Session_Manager;
use Growmatik\WooCommerce\Commerce_Data_Layer;

/**
 * Class User_Order_Controller
 *
 * Any functionality related to WooCommerce orders management should be in this class.
 *
 * IMPORTANT: Due to 1001 in all parts all order statuses (whether paid or unpaid) must be considered in calculations
 * and determining the user type.
 *
 * @package Growmatik\Controller
 */
class User_Order_Controller {
	/**
	 * refer -> wc_customer_bought_product
	 *
	 * @param mixed $email
	 * @return bool
	 */
	public function user_has_ordered_by_email( $email ) {
		$user = get_user_by( 'email', $email );

		if ( ! $user ) {
			return false;
		}

		return $this->user_has_ordered_by_user( $user->ID );
	}

	public function get_user_order_count( $user_id ) {
		if ( ! \Growmatik\is_woocommerce_active() ) {
			return 0;
		}

		if ( ! $user_id ) {
			return 0;
		}

		/**
		 * Check if user has order or not.
		 *
		 * Has builtin cache (user meta).
		 * @link https://docs.woocommerce.com/wc-apidocs/source-class-WC_Customer_Data_Store.html#349-377
		 */
		return wc_get_customer_order_count( $user_id );
	}

	public function user_has_ordered_by_user( $user_id ) {
		$result = $this->get_user_order_count( $user_id );

		return $result > 0;
	}

	/**
	 * Returns a list of purchased product_ids for the current user, based on all order statuses.
	 *
	 * - If the current user is logged in, return based on her user id; however since we associate guest customers with
	 *   users this will effectively include guest checkouts. (After Unified Subscription we no longer associate and the
	 *   logic may need adjustment.)
	 * - If the current user is a guest customer (not logged-in), return guest checkouts based on billing email; which
	 *   effectively includes the purchases for the logged-in times.
	 *
	 * Inspired by:
	 * - wc_customer_bought_product
	 * - WC_Customer_Data_Store functions using _customer_user, especially get_total_spent
	 * - https://rudrastyh.com/woocommerce/display-purchased-products.html
	 * - We preferred double-joins over https://www.businessbloomer.com/woocommerce-display-products-purchased-user/
	 *
	 * @return array
	 */
	public function list_of_purchased_product_ids() {
		if ( ! \Growmatik\is_woocommerce_active() ) {
			return array();
		}

		global $wpdb;

		$cache_key = 'products_list';

		$user_id = null;
		$email   = null;

		if ( 'guest' !== \Growmatik\get_user_type() ) {
			$user_id = get_current_user_id();

			if ( $this->has_user_cache( $user_id, $cache_key ) ) {
				return $this->get_user_cache( $user_id, $cache_key ) ?: array();
			}

			$variable   = $user_id;
			$sql_filter = "ordermeta.meta_key = '_customer_user' AND ordermeta.meta_value = %s";
		} else {
			$session = new Session_Manager();
			$email   = $session->get_effective_user_property( 'email' );

			if ( ! $email ) {
				return array();
			}

			if ( $this->has_guest_cache( $email, $cache_key ) ) {
				return $this->get_guest_cache( $email, $cache_key ) ?: array();
			}

			$variable   = $email;
			$sql_filter = "ordermeta.meta_key = '_billing_email' AND ordermeta.meta_value = %s";
		}

		$statuses_list       = array_keys( wc_get_order_statuses() );
		$status_placeholders = implode( ', ', array_fill( 0, count( $statuses_list ), '%s' ) );

		$sql_binds = array_merge( $statuses_list, array( $variable ) );

		// phpcs:disable WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.PreparedSQLPlaceholders.UnfinishedPrepare
		$purchased_products_ids = $wpdb->get_col(
			$wpdb->prepare(
				"
				SELECT      itemmeta.meta_value
				FROM        {$wpdb->prefix}woocommerce_order_itemmeta itemmeta
				INNER JOIN  {$wpdb->prefix}woocommerce_order_items items
				            ON itemmeta.order_item_id = items.order_item_id
				INNER JOIN  {$wpdb->posts} orders
				            ON orders.ID = items.order_id
				INNER JOIN  {$wpdb->postmeta} ordermeta
				            ON orders.ID = ordermeta.post_id
				WHERE       orders.post_status IN ( {$status_placeholders} )
							AND itemmeta.meta_key = '_product_id'
				            AND {$sql_filter}
				ORDER BY    orders.post_date DESC
				",
				$sql_binds
			)
		);
		// phpcs:enable

		$products_list = array_values( array_unique( $purchased_products_ids ?: array() ) );

		// Front-end 'Purchased Product' rule execution expects these to be strings, ensure they are strings
		$products_list = array_map( 'strval', $products_list );

		if ( 'guest' !== \Growmatik\get_user_type() ) {
			$this->set_user_cache( $user_id, $cache_key, $products_list );
		} else {
			$this->set_guest_cache( $email, $cache_key, $products_list );
		}

		return $products_list;
	}

	/**
	 * Get current user or guest last order payment method.
	 *
	 * @return array
	 */
	public function get_last_order() {
		$order = Commerce_Data_Layer::empty_order( 'woo' );

		if ( ! \Growmatik\is_woocommerce_active() ) {
			return $order;
		}

		$cache_key = 'last_order';

		if ( 'guest' !== \Growmatik\get_user_type() ) {
			$user_id = get_current_user_id();

			if ( $this->has_user_cache( $user_id, $cache_key ) ) {
				return $this->get_user_cache( $user_id, $cache_key ) ?: $order;
			}
		} else {
			// Effective user ID for guest-customer.
			$session = new Session_Manager();
			$user_id = $session->get_effective_user_property( 'userId' );
			$email   = $session->get_effective_user_property( 'email' );

			if ( ! $email ) {
				return $order;
			}
			if ( $this->has_guest_cache( $email, $cache_key ) ) {
				return $this->get_guest_cache( $email, $cache_key ) ?: $order;
			}
		}

		$last_order = wc_get_customer_last_order( $user_id );
		// Order didn't exists.
		if ( ! $last_order ) {
			return $order;
		}

		$order = Commerce_Data_Layer::get_order_details_for_data_layer( $last_order );

		if ( 'guest' !== \Growmatik\get_user_type() ) {
			$this->set_user_cache( $user_id, $cache_key, $order );
		} else {
			$this->set_guest_cache( $email, $cache_key, $order );
		}

		return $order;
	}

	/**
	 * Get current user total purchased items from multiple orders.
	 *
	 * TODO: Duplicate of $this->list_of_purchased_product_ids() method.
	 *
	 * @return integer
	 */
	public function get_total_purchased_items() {
		if ( ! \Growmatik\is_woocommerce_active() ) {
			return 0;
		}

		global $wpdb;

		$cache_key = 'total_purchased_items';

		$user_id = null;
		$email   = null;

		if ( 'guest' !== \Growmatik\get_user_type() ) {
			$user_id = get_current_user_id();

			if ( $this->has_user_cache( $user_id, $cache_key ) ) {
				return (int) $this->get_user_cache( $user_id, $cache_key ) ?: 0;
			}

			$variable   = $user_id;
			$sql_filter = "ordermeta.meta_key = '_customer_user' AND ordermeta.meta_value = %s";
		} else {
			$session = new Session_Manager();
			$email   = $session->get_effective_user_property( 'email' );

			if ( ! $email ) {
				return 0;
			}

			if ( $this->has_guest_cache( $email, $cache_key ) ) {
				return (int) $this->get_guest_cache( $email, $cache_key ) ?: 0;
			}

			$variable   = $email;
			$sql_filter = "ordermeta.meta_key = '_billing_email' AND ordermeta.meta_value = %s";
		}

		$statuses_list       = array_keys( wc_get_order_statuses() );
		$status_placeholders = implode( ', ', array_fill( 0, count( $statuses_list ), '%s' ) );

		$sql_binds = array_merge( $statuses_list, array( $variable ) );

		// phpcs:disable WordPress.DB.PreparedSQL.InterpolatedNotPrepared, WordPress.DB.PreparedSQLPlaceholders.UnfinishedPrepare
		$total_purchased_items = (int) $wpdb->get_var(
			$wpdb->prepare(
				"
				SELECT      COUNT(itemmeta.meta_value)
				FROM        {$wpdb->prefix}woocommerce_order_itemmeta itemmeta
				INNER JOIN  {$wpdb->prefix}woocommerce_order_items items
				            ON itemmeta.order_item_id = items.order_item_id
				INNER JOIN  {$wpdb->posts} orders
				            ON orders.ID = items.order_id
				INNER JOIN  {$wpdb->postmeta} ordermeta
				            ON orders.ID = ordermeta.post_id
				WHERE       orders.post_status IN ( {$status_placeholders} )
							AND itemmeta.meta_key = '_product_id'
				            AND {$sql_filter}
				ORDER BY    orders.post_date DESC
				",
				$sql_binds
			)
		);

		if ( 'guest' !== \Growmatik\get_user_type() ) {
			$this->set_user_cache( $user_id, $cache_key, $total_purchased_items );
		} else {
			$this->set_guest_cache( $email, $cache_key, $total_purchased_items );
		}

		return (int) $total_purchased_items;
	}

	/**#@+
	 * Functions for working with logged-in users' commerce cache.
	 */
	const META_PREFIX = '_' . self::OPTION_PREFIX;

	private function has_user_cache( $user_id, $key ) {
		return get_user_meta( $user_id, self::META_PREFIX . $key, true ) !== '';
	}

	private function get_user_cache( $user_id, $key ) {
		return get_user_meta( $user_id, self::META_PREFIX . $key, true );
	}

	private function set_user_cache( $user_id, $key, $value ) {
		update_user_meta( $user_id, self::META_PREFIX . $key, $value );
	}

	public static function invalidate_user_cache( $user_id ) {
		if ( ! $user_id ) {
			return; // Be safe
		}

		global $wpdb;

		$wpdb->query(
			"DELETE FROM $wpdb->usermeta WHERE user_id = " . esc_sql( $user_id ) .
			" AND meta_key LIKE '" . self::META_PREFIX . "%'" // phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		);
	}
	/**#@-*/

	/**#@+
	 * Functions for working with guest customers' commerce cache.
	 */
	const OPTION_PREFIX = 'growmatik_ecomcache_';

	private function has_guest_cache( $email, $key ) {
		return get_transient( self::OPTION_PREFIX . "$key:$email" ) !== false;
	}

	private function get_guest_cache( $email, $key ) {
		return get_transient( self::OPTION_PREFIX . "$key:$email" );
	}

	private function set_guest_cache( $email, $key, $value ) {
		set_transient( self::OPTION_PREFIX . "$key:$email", $value, WEEK_IN_SECONDS );
	}

	public static function invalidate_guest_cache( $email ) {
		global $wpdb;
		$pattern = "'%" . self::OPTION_PREFIX . '%' . esc_sql( $email ) . "%'";

		// phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		$wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE $pattern" );
	}

	public static function remove_data() {
		global $wpdb;

		// phpcs:ignore WordPress.DB.PreparedSQL.NotPrepared
		$wpdb->query( "DELETE FROM $wpdb->options WHERE option_name LIKE '%" . self::OPTION_PREFIX . "%'" );
	}
	/**#@-*/
}
