<?php

namespace Growmatik\EDD;

use EDD_Customer;
use EDD_Payment;
use Growmatik\Event_Transport;
use Growmatik\Modification_Handler;
use Growmatik\Session_Manager;

class Checkout {

	const CHECKOUT_HANDLED_META = '_growmatik_checkout_handled';

	/** @var Session_Manager */
	private $session;

	private $user = false;

	/** @var EDD_Payment */
	private $order;

	/** @var EDD_Customer */
	private $customer;


	public function __construct() {
		// Was not possible using `edd_complete_purchase` or `edd_payment_receipt_before` hooks.
		// Refer to the commit docs for the reason.
		add_action( 'template_redirect', array( $this, 'safe_handle_checkout' ) );
	}

	public function safe_handle_checkout() {
		if ( ! edd_is_success_page() ) {
			return;
		}

		$session    = edd_get_purchase_session();
		$payment_id = 0;

		if ( $session ) {
			$payment_key = $session['purchase_key'];
			$payment_id  = edd_get_purchase_id_by_key( $payment_key );
		}

		if ( ! $payment_id ) {
			return;
		}

		$this->order    = new EDD_Payment( $payment_id );
		$this->customer = new EDD_Customer( $this->order->customer_id );

		try {
			$this->handle_checkout();
		} catch ( \Throwable $ex ) { //phpcs:ignore Generic.CodeAnalysis.EmptyStatement.DetectedCatch
			// Record $ex using Sentry
		}
	}

	public function handle_checkout() {
		$this->session = new Session_Manager();

		$this->find_existing_user()
			->record_order_meta_data()
			->reset_session_id()
			->override_user_type()
			->invalidate_ecommerce_cache()
			->fixup_data_layer_commerce_details();

		if ( ! $this->is_checkout_handled() ) {
			Modification_Handler::handle_checkout( $this->order );
			$this->send_checkout_event()
				->mark_checkout_handled();
		}
	}

	/**
	 * Finds and stores the existing user of the person that did a checkout.
	 *
	 * If the user is logged in do not use the billing email, doing so results in the logged-in user being considered a
	 * guest if they enter a different email address than their WordPress email on WooCommerce Checkout page.
	 *
	 * @return $this
	 */
	private function find_existing_user() {
		if ( 'guest' === \Growmatik\get_user_type() ) {
			$this->user = get_user_by( 'email', $this->customer->email );

			if (
				false !== $this->user &&
				is_multisite() &&
				! is_user_member_of_blog( $this->user->ID, get_current_blog_id() )
			) {
				$this->user = false;
			}
		} else {
			$this->user = wp_get_current_user();
		}

		return $this;
	}

	/**
	 * Record session and client information for the order
	 *
	 * @return $this
	 */
	public function record_order_meta_data() {
		$meta_prefix = \Growmatik\meta_site_prefix();

		if ( $this->order->get_meta( "{$meta_prefix}_session_id" ) ) {
			return $this;
		}

		$this->order->add_meta( "{$meta_prefix}_session_id", $this->session->get_session_id(), true );
		$this->order->add_meta( "{$meta_prefix}_client_id", $this->session->get_client_id(), true );
		$this->order->save();

		return $this;
	}

	/**
	 * Resets the session ID after purchase. The session ID will not change in this request.
	 *
	 * @return $this
	 */
	private function reset_session_id() {
		// We just want to reset the exact session the order was made in; So we must not use `$session->get_session_id`
		// because that will reset the session each time the user visits the thankyou page.
		$meta_prefix = \Growmatik\meta_site_prefix();
		$session_id  = $this->order->get_meta( "{$meta_prefix}_session_id" );

		$this->session->reset_session_id( $session_id, 'edd_checkout' );

		return $this;
	}

	/**
	 * Determine if this is a Guest Checkout.
	 *
	 * Use this function because our definition of 'guest' is different from a WordPress logged in user WRT users that
	 * are a member of another blog on multi-site, and an existing member that logs out and buys things with the same
	 * email.
	 *
	 * @return bool
	 */
	private function is_guest_checkout() {
		return ( false === $this->user );
	}

	/**
	 * Overrides the user type. This override will be picked up by FE.
	 *
	 * @return $this
	 */
	private function override_user_type() {
		// In this specific function do not use is_guest_checkout due to 593/594: When people* are logged out we'd like
		// to temporarily upgrade their current session anyway. (*People above includes own leads/customers and aliens.)
		if ( 'guest' !== \Growmatik\get_user_type() ) {
			return $this;
		}

		$this->session->override_user_props(
			array(
				'type'         => 'customer',
				'email'        => $this->customer->email,
				'username'     => $this->customer->email,
				'firstName'    => $this->customer->name,
				'lastName'     => $this->customer->name,
				'registeredAt' => $this->customer->date_created,
			)
		);

		$last_order = Commerce_Data_Layer::get_order_details_for_data_layer( $this->order );

		$this->session->override_commerce_props(
			'edd',
			array(
				'cartItems'          => array(),
				'hasCart'            => false,
				'lastOrder'          => $last_order,
				'noOfOrders'         => 1,
				'noOfPurchasedItems' => (int) $last_order['itemCount'],
				'products'           => array_map( 'strval', array_keys( $last_order['items'] ) ),
			)
		);

		return $this;
	}

	/**
	 * Invalidates the e-commerce cache for the user or guest customer.
	 *
	 * @return $this
	 */
	private function invalidate_ecommerce_cache() {
		if ( $this->user ) {
			User_Orders::invalidate_user_cache( $this->user->ID );
		}

		User_Orders::invalidate_guest_cache( $this->customer->email );

		return $this;
	}

	public function fixup_data_layer_commerce_details() {
		$orders = new User_Orders();
		add_filter( 'growmatik_user_purchased_product_ids_edd', array( $orders, 'list_of_purchased_product_ids' ) );
		add_filter( 'growmatik_user_total_purchased_items_edd', array( $orders, 'get_total_purchased_items' ) );
	}

	/**
	 * Reports a checkout event to backend.
	 *
	 * @return $this
	 */
	private function send_checkout_event() {
		// Select only main order data.
		$order_data = (object) $this->order->array_convert();

		// Plus the id.
		$order_data->id = $this->order->ID;

		// Prepare and send the event.
		$event_name = $this->is_guest_checkout() ? 'wpGuestCheckout' : 'wpUserCheckout';
		$event_data = array(
			'order' => $order_data,
		);

		// If user is not logged in but her account already exists, it means she is now a customer and WooCommerce will
		// -or- already made her a member of the current blog in multi-site.  However Event_Transport still thinks of
		// her as a Guest, so we force the user type.
		$event_user_id   = $this->is_guest_checkout() ? null : $this->user->ID;
		$event_user_type = $this->is_guest_checkout() ? null : 'customer';

		$transport = new Event_Transport( 'edd' );
		$transport->send_event( $event_name, $event_data );

		return $this;
	}

	/**
	 * Checks if the checkout is handled.
	 *
	 * @return bool
	 */
	private function is_checkout_handled() {
		return (bool) $this->order->get_meta( self::CHECKOUT_HANDLED_META );
	}

	/**
	 * Sets metadata on the order specifying the checkout was handled.
	 *
	 * @return $this
	 */
	private function mark_checkout_handled() {
		$this->order->add_meta( self::CHECKOUT_HANDLED_META, '1', true );
		$this->order->save();

		return $this;
	}
}
