<?php


namespace Growmatik;

use Growmatik\Controller\User_Order_Controller;

function get_user_type() {
	if ( ! is_user_logged_in() ) {
		return 'guest';
	}

	if ( is_multisite() ) {
		if ( ! is_user_member_of_blog( get_current_user_id(), get_current_blog_id() ) ) {
			return 'guest';
		}
	}

	$user_types = array();

	if ( is_woocommerce_active() ) {
		$orders = new User_Order_Controller();

		$user_types[] = $orders->get_user_order_count( get_current_user_id() ) > 0 ? 'customer' : 'lead';
	}

	if ( is_edd_active() ) {
		$customer     = new \EDD_Customer( get_current_user_id() );
		$user_types[] = ( $customer->purchase_count > 0 ) ? 'customer' : 'lead';
	}

	return in_array( 'customer', $user_types, true ) ? 'customer' : 'lead';

}

/**
 * Retrieves the site verification token if not available, and constructs a user identity token using it.
 *
 * @param int $user_id
 * @return string
 */
function get_user_identity_token( $user_id ) {
	$verification_secret = get_option( 'growmatik_verification_secret' );

	if ( ! $verification_secret ) {
		$verification = GraphQL_Client::make()->identity_verification_secret()->execute();
		if ( is_array( $verification ) && ! empty( $verification['value'] ) ) {
			update_option( 'growmatik_verification_secret', $verification['value'] );
			$verification_secret = $verification['value'];
		}
	}

	return ( $user_id && $verification_secret )
		? hash_hmac( 'sha256', $user_id, $verification_secret )
		: '';
}

function is_woocommerce_active() {
	return did_action( 'woocommerce_loaded' );
}

function starts_with( $haystack, $needle ) {
	return substr( $haystack, 0, strlen( $needle ) ) === $needle;
}

function error_log( $log ) {
	if ( is_array( $log ) || is_object( $log ) ) {
		\error_log( print_r( $log, true ) ); // phpcs:ignore
		return;
	}

	\error_log( $log ); // phpcs:ignore
}

/**
 * Checks whether $search is an array and all $keys exist in it, whether null or non-null.
 *
 * @param array $keys
 * @param array $search
 *
 * @return bool
 */
function array_keys_exist( $keys, $search ) {
	if ( ! is_array( $search ) ) {
		return false;
	}

	$count = 0;
	foreach ( $keys as $key ) {
		$count += ( isset( $search[ $key ] ) || array_key_exists( $key, $search ) ) ? 1 : 0;
	}

	return count( $keys ) === $count;
}

/**
 * Returns a string to be used as the meta key prefix for Growmatik meta entries in places where the meta must only be
 * available for a specific site and must not be synced to other sites. As an example if tenant re-integrates with a new
 * site id, we don't want to use the old site's purchase session ids with the new site. Therefore append site_id to
 * order metas and other metas which should not survive on changing sites.
 *
 * @return string
 */
function meta_site_prefix() {
	$site_id = get_option( 'growmatik_site_id' );
	return "_growmatik_{$site_id}";
}

function get_relative_permalink( $post ) {
	$permalink = get_permalink( $post );
	$url       = str_replace( home_url(), '', $permalink );

	return rtrim( $url, '/' );
}

function is_screen( $screen_id ) {
	$screen = get_current_screen();
	if ( ! $screen ) {
		return false;
	}

	return $screen_id === $screen->id;
}

/**
 * Utility function to verify nonces without causing warnings in WPCS.
 *
 * @param string $action
 *
 * @param string $field_name
 *
 * @return bool
 */
function verify_get_nonce( $action, $field_name = 'nonce' ) {
	if ( ! isset( $_GET[ $field_name ] ) ) {
		return false;
	}

	// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
	if ( ! wp_verify_nonce( wp_unslash( $_GET[ $field_name ] ), $action ) ) {
		return false;
	}

	return true;
}

/**
 * Utility function to verify nonces without causing warnings in WPCS.
 *
 * @param string $action
 *
 * @param string $field_name
 *
 * @return bool
 */
function verify_post_nonce( $action, $field_name = 'nonce' ) {
	if ( ! isset( $_POST[ $field_name ] ) ) {
		return false;
	}

	// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
	if ( ! wp_verify_nonce( wp_unslash( $_POST[ $field_name ] ), $action ) ) {
		return false;
	}

	return true;
}

/**
 * Checks for main class of Easy Digital Downloads plugin.
 *
 * @return bool True if main class of plugin exists.
 */
function is_edd_active() {
	return class_exists( 'Easy_Digital_Downloads' );
}
